//
//  SessionData.swift
//  MotionDataLogging
//
//  Created by Fabiano Finocchio on 17/05/2019.
//  Copyright © 2019 Fabiano Finocchio. All rights reserved.
//

import Foundation

struct SessionData : Codable {
    
    private var label : String?
    var device : String?
    private var frequency: Int?
    private var accelerationX : [Double]
    private var accelerationY : [Double]
    private var accelerationZ : [Double]
    private var gyroX : [Double]
    private var gyroY : [Double]
    private var gyroZ : [Double]
    private var pitch : [Double]
    private var roll : [Double]
    private var yaw : [Double]
    private var locatioUpdates : [locationUpdates]
    private var heartUpdates : [heartValues]
    private var startDataAcquisition : Date?
    private var stopDataAcquisition : Date?
    private var startAcquisitionString : String?
    private var stopAcquistionString : String?
    private var durationAcquisition : Double?
    
    
    
    var json: Data {
        return try! JSONEncoder().encode(self)
    }
    
    init() {
        self.accelerationX = [Double]()
        self.accelerationY = [Double]()
        self.accelerationZ = [Double]()
        self.gyroX = [Double]()
        self.gyroY = [Double]()
        self.gyroZ = [Double]()
        self.pitch = [Double]()
        self.roll = [Double]()
        self.yaw = [Double]()
        self.locatioUpdates = [locationUpdates]()
        self.heartUpdates = [heartValues]()
        self.frequency = 50;
        
    }
    
    
    
    func getLabel()->String{
        if self.label == nil{
            return "None"
        }else {
            return self.label!
        }
    }
    mutating func setLabel (label: String){
        self.label = label
    }
    mutating func setDevice( device: String){
        self.device = device
    }
    func getAccX()->[Double]{
        return self.accelerationX
    }
    func getAccZ()->[Double]{
        return self.accelerationZ
    }
    func getAccY()->[Double]{
        return self.accelerationY
    }
    func getGyroX()->[Double]{
        return self.gyroX
    }
    func getGyroY()->[Double]{
        return self.gyroY
    }
    func getGyroZ()->[Double]{
        return self.gyroZ
    }
    func getPitch()->[Double]{
        return self.pitch
    }
    func getYaw()->[Double]{
        return self.yaw
    }
    func getRoll()->[Double]{
        return self.roll
    }
    
    func getStart()->Date{
        return self.startDataAcquisition!
    }
    func getStop()->Date{
        return self.stopDataAcquisition!
    }
    func getFrequency()->Int{
        return self.frequency!
        
    }
    mutating func setFrequency(frequency : Int){
        self.frequency = frequency
        
    }
    
    mutating func setStartAcquisition() {
        self.startDataAcquisition = Date()
        self.startAcquisitionString = convertDateToString(date: self.startDataAcquisition!)
    }
    mutating func setStopAcquisition(){
        self.stopDataAcquisition = Date()
        self.stopAcquistionString = convertDateToString(date: self.stopDataAcquisition!)
    }
    mutating func appendGyroValues (x: Double, y: Double, z:Double){
        self.gyroX.append(x)
        self.gyroY.append(y)
        self.gyroZ.append(z)
        
    }
    
    mutating func appendAccValues(x:Double, y: Double, z:Double){
        self.accelerationX.append(x)
        self.accelerationY.append(y)
        self.accelerationZ.append(z)
    }

    
    mutating func appendOrientationValues (pitch : Double, yaw : Double, roll : Double){
        self.roll.append(roll)
        self.yaw.append(yaw)
        self.pitch.append(pitch)
    }
    
    
    mutating func appendLocationValues (locationUpdate: locationUpdates ){
        self.locatioUpdates.append(locationUpdate)
    }
    mutating func appendHeartValues (heartValue : heartValues){
        self.heartUpdates.append(heartValue)
    }
    
    
    func convertDateToString (date:Date) -> String {
        let formatter = DateFormatter()
        formatter.dateFormat = "yyyy-MM-dd_HH:mm:ss"
        let myString = formatter.string(from: date)
        return myString
        
    }
    
    func getDocumentName(number:Int)->String{
        var documentName : String
        documentName = "DataSession_\(device!)_\(self.getLabel())_\(number).json"
        return documentName
    }
    
    func saveData(number: Int){
        var documentName : String
        
        documentName = "DataSession_\(device!)_\(self.getLabel())_\(number).json"
        
        Storage.store(self, to: .documents, as: documentName )
        if Storage.fileExists(documentName, in: .documents){
            print("Saved data successfully")
        }
        else{
            print ("Something went wrong")
        }
    }
    
    mutating func clearAll(){
        self.startDataAcquisition = nil
        self.stopDataAcquisition = nil
        self.startAcquisitionString = nil
        self.stopAcquistionString = nil
        self.label = nil
        self.accelerationX.removeAll()
        self.accelerationY.removeAll()
        self.accelerationZ.removeAll()
        self.gyroZ.removeAll()
        self.gyroY.removeAll()
        self.gyroX.removeAll()
        self.pitch.removeAll()
        self.yaw.removeAll()
        self.roll.removeAll()

        self.locatioUpdates.removeAll()
        self.heartUpdates.removeAll()
    }
    
}

struct locationUpdates : Codable {
    var currentSpeed : Double
    var timeStampAcquistion : Date
    
    init( currentSpeed : Double, timeStampAcquistion : Date){
        self.currentSpeed = currentSpeed
        self.timeStampAcquistion = timeStampAcquistion
    }
    
}

struct heartValues : Codable {
    var heartRate : Double
    var timeStampAcquistion : Date
    
    init(heartRate : Double, timeStampAcquistion : Date) {
        self.heartRate = heartRate
        self.timeStampAcquistion = timeStampAcquistion
    }
}

